*  Program Ŀ
*   Application: MY_MENU Demo                                               
*   Description: Data driven pull down menu system                          
*     File Name: MY_MENU.PRG                                                
*        Author: Steven Gibb                                                
*  Date created: 09-15-93              Date updated: 09-15-93              
*  Time created: 11:44:43am            Time updated: 11:44:43am            
*     Make File: MY_MENU.RMK                                                
*     Exec File: MAIN.EXE              Docs By: Steven Gibb                 
*     Copyright: (c) 1993 by Steven Gibb                                    
* 

#include "inkey.ch"
#include "set.ch"
#include "my_menu.ch"

#define  MENU_STRUCT        aMenuData
#define  NBR_MENUS          aMenuData[1]
#define  BAR_PAD            aMenuData[2]
#define  START_COL          aMenuData[3]
#define  BAR_MENU_SCREEN    aMenuData[4]
#define  BAR_HOTKEYS        aMenuData[5]
#define  BAR_CHOICES        aMenuData[6]
#define  LEFT_COL           aMenuData[7]
#define  SPACE_BETWEEN      aMenuData[8]
#define  MENU_ARRAY         aMenuData[9]
#define  MENU_WRAP          aMenuData[10]
#define  MENU_COLOR         aMenuData[11]
#define  BACKGROUND_COLOR   aMenuData[12]
#define  HIGHLIGHT          aMenuData[13]
#define  MSG_COLOR          aMenuData[14]
#define  HOTKEY_COLOR       aMenuData[15]
#define  SHADOW             aMenuData[16]
#define  MENU_MONO          aMenuData[17]
#define  HIGHLT_MONO        aMenuData[18]
#define  MSG_MONO           aMenuData[19]
#define  HOTKEY_MONO        aMenuData[20]
#define  BACKGROUND_MONO    aMenuData[21]
#define  ARRAY_SIZE         21

#define  ITEMS_ACTIVE_STATUS  aSelStatus

#define  SUBMENU_SCREEN     SUBMENU_ARRAY[1]     //  SAVESCREEN
#define  SUBMENU_BOT_ROW    SUBMENU_ARRAY[2]     //  iSubHeight+1
#define  SUBMENU_ITEM_LEN   SUBMENU_ARRAY[3]     //  iSubWidth+1
#define  SUBMENU_PROMPTS    SUBMENU_ARRAY[4]     //  aSubPrompts
#define  SUBMENU_ACTIONS    SUBMENU_ARRAY[5]     //  aSubActions
#define  SUBMENU_MSGS       SUBMENU_ARRAY[6]     //  aSubMsgs
#define  SUBMENU_HOTKEYS    SUBMENU_ARRAY[7]     //  cHotKeys
#define  SUBMENU_ELE_NBR    SUBMENU_ARRAY[8]     //  aItemNbrs

#define  TRUE               .T.
#define  FALSE              .F.

#define  MENU_STATUS        aMenuStatus
#define  TOP_BAR_POSITION   aMenuStatus[1]
#define  TOP_BAR_LAST_POS   aMenuStatus[2]
#define  SUB_LEVEL_DEPTH    aMenuStatus[3]
#define  ITEM_IS_ACTIVE     aMenuStatus[4]
#define  SELECTED_ITEM      aMenuStatus[5]


#define  SUBMENU_ARRAY      MENU_ARRAY[TOP_BAR_POSITION][2]
#define  SUBMENU_COL        LEFT_COL[TOP_BAR_POSITION]
#define  BAR_ITEM           BAR_CHOICES[TOP_BAR_POSITION]


STATIC MENU_STRUCT
STATIC MENU_STATUS := { 1, 99, 0, TRUE, 1 }

STATIC aScreenStack := {}
STATIC iScreenStackNbr := 0
STATIC ITEMS_ACTIVE_STATUS := {}

*  Function Ŀ
*          Name: DisplayMenu()         Docs: Steven Gibb                    
*   Description: Operates menu system                                       
*        Author: Steven Gibb                                                
*  Date created: 09-15-93              Date updated: 09-15-93              
*  Time created: 11:49:23am            Time updated: 11:49:23am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: aMenu  - ragged array that stores all data to run menus    
*  Return Value: ( NIL )                                                    
*      See Also: InitMenu()                                                 
* 
FUNCTION DisplayMenu( aMenu )
   LOCAL bChoice

   // connects array reference passed to this function to file wide STATIC
   MENU_STRUCT := aMenu

   // switch to monochrome values if not color system
   CheckColor()

   SETCURSOR(0)

   // loop here until program quits
   DO WHILE TRUE
      MEMORY(-1)
      bChoice := GetTopLevelPullDownAction()
      IF VALTYPE(bChoice)==[B]  .AND. ITEM_IS_ACTIVE
         EVAL(bChoice)
      ENDIF
   ENDDO

RETURN( NIL )

*  Function Ŀ
*          Name: GetTopLevelPullDownAction()                                
*   Description: Display all changes to top bar menu and top level pulldown 
*                menus.  Return code block associated with menu item on a   
*                top level menu item when selected to be executed, otherwise
*                return NIL.                                                
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:18:12pm            Time updated: 07:18:12pm            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: none                                                       
*  Return Value: ( bReturn )                                                
*      See Also:                                                            
* 
STATIC FUNCTION GetTopLevelPullDownAction( )
   LOCAL bReturn, br := SUBMENU_BOT_ROW + 1

   SET CURSOR OFF

   DISPBEGIN()

   // IF either initial pulldown display or top level
   // pulldown changed, draw new menu screen display
   IF TOP_BAR_POSITION <> TOP_BAR_LAST_POS

      // restore top line bar
      RESTSCREEN(00,00,00,MAXCOL(), BAR_MENU_SCREEN)


      // highlight current top bar selection
      @ 00, SUBMENU_COL SAY BAR_ITEM  COLOR HIGHLIGHT


      // clear old pulldown display
      @ 01,00,22,79 BOX [] COLOR BACKGROUND_COLOR

      // re-Draw shadow for top bar menu
      IF SHADOW
         ShadowStrip(1,0,1,77)
      ENDIF

      // if top menu bar changed, and move the item selected pointer
      // up if below the bottom of new pulldown choices
      SELECTED_ITEM := IF( SELECTED_ITEM > br-2, br-2, SELECTED_ITEM )

      // reset top bar postion
      TOP_BAR_LAST_POS := TOP_BAR_POSITION

   ENDIF

   DISPEND()

   BEGIN SEQUENCE

      // call routine to display and operate pulldown choices
      SELECTED_ITEM := ProcessKeysForTopLevelMenu()

      // if top pulldown menu choice was requested
      IF SELECTED_ITEM > 0

         bReturn :=  SUBMENU_ACTIONS[ SELECTED_ITEM ]

      ENDIF
   END SEQUENCE

RETURN( bReturn  )


*  Function Ŀ
*          Name: ProcessKeysForTopLevelMenu()                               
*   Description: Process keystrokes when operating on the top menu level    
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:26:07pm            Time updated: 07:26:07pm            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: None                                                       
*  Return Value: ( iSelectPos )                                             
*      See Also:                                                            
* 
FUNCTION ProcessKeysForTopLevelMenu()
   LOCAL iSelectPos  := SELECTED_ITEM
   LOCAL iKey, cKey
   LOCAL iRCol    := SUBMENU_COL + SUBMENU_ITEM_LEN
   LOCAL iBRow    := SUBMENU_BOT_ROW +1

   // stay in this loop until a menu item action is selected to be
   // executed or a keystroke is entered that changes to another
   // top level menu
   WHILE TRUE
      DrawPullDownMenu(  1,;
         SUBMENU_COL,;
         iBRow,;
         iRCol,;
         SUBMENU_SCREEN,;
         iSelectPos,;
         SUBMENU_PROMPTS,;
         SUBMENU_MSGS,;
         SUBMENU_ELE_NBR)

      iKey := InKeyWait(0)
      cKey := UPPER( CHR(iKey))
      DO CASE
         CASE iKey == K_HOME
            iSelectPos := 1
         CASE iKey == K_END
            iSelectPos := SUBMENU_BOT_ROW  -1
         CASE iKey == K_UP
            IF iSelectPos == 1
               IF MENU_WRAP
                  iSelectPos := SUBMENU_BOT_ROW  -1
               ENDIF
            ELSE
               iSelectPos--
            ENDIF
         CASE iKey == K_DOWN
            IF iSelectPos == SUBMENU_BOT_ROW -1
               IF MENU_WRAP
                  iSelectPos := 1
               ENDIF
            ELSE
               iSelectPos++
            ENDIF

         CASE iKey == K_RIGHT
            TOP_BAR_POSITION++
            IF TOP_BAR_POSITION > NBR_MENUS
               TOP_BAR_POSITION := IF( MENU_WRAP , 1, --TOP_BAR_POSITION)
            ENDIF

            BREAK

         CASE iKey == K_LEFT
            TOP_BAR_POSITION--
            IF TOP_BAR_POSITION < 1
               TOP_BAR_POSITION := IF( MENU_WRAP, NBR_MENUS, ++TOP_BAR_POSITION)
            ENDIF
            BREAK

         CASE iKey == K_ESC
            TOP_BAR_POSITION := LEN(BAR_HOTKEYS)
            BREAK

         CASE cKey $ BAR_HOTKEYS
            TOP_BAR_POSITION := AT( cKey,BAR_HOTKEYS )
            BREAK

         CASE cKey $ SUBMENU_HOTKEYS
            iSelectPos := AT( cKey, SUBMENU_HOTKEYS )
            EXIT

         CASE iKey == K_RETURN
            EXIT
      ENDCASE
      SELECTED_ITEM := iSelectPos
   ENDDO

   SELECTED_ITEM := iSelectPos

   DrawPullDownMenu(  1,;
      SUBMENU_COL,;
      iBRow,;
      iRCol,;
      SUBMENU_SCREEN,;
      iSelectPos,;
      SUBMENU_PROMPTS,;
      SUBMENU_MSGS,;
      SUBMENU_ELE_NBR)


RETURN( iSelectPos )


*  Function Ŀ
*          Name: DrawPullDownMenu()      Docs: Steven Gibb                  
*   Description: Restore to the screen in the correct location, the pulldown
*                menu screen saved during menu initialization. Also redraw  
*                the menu prompt at cursor location (highlighted) in grey if
*                menu item NOT active and W+/R if menu item is active.      
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 10:12:12am            Time updated: 10:12:12am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: iTopRow                                                    
*              : iLCol                                                      
*              : iBRow                                                      
*              : iRCol                                                      
*              : cPullDownScreen                                            
*              : iPositionOfHighLightedItem                                 
*              : aPromptsForPullDown                                        
*              : aMsgsForPullDown                                           
*              : aSeqNbrsForPullDownItems                                               
*  Return Value: (NIL)                                                      
*      See Also:                                                            
* 
STATIC FUNCTION DrawPullDownMenu(iTopRow,iLCol,iBRow,iRCol,;
      cPullDownScreen,;
      iPositionOfHighLightedItem,;
      aPromptsForPullDown,;
      aMsgsForPullDown,;
      aSeqNbrsForPullDownItems )

   DISPBEGIN()
   RESTSCREEN(iTopRow,iLCol,iBRow,iRCol,cPullDownScreen)

   IF SHADOW
      ShadowStrip(iTopRow+1,iLCol-2,iBRow+1,iLCol-1  )
      ShadowStrip(iBRow+1,iLCol,iBRow+1,iRCol-2  )
   ENDIF


   ITEM_IS_ACTIVE := ITEMS_ACTIVE_STATUS[ ;
      aSeqNbrsForPullDownItems[ iPositionOfHighLightedItem ];
                                          ]

   @ iTopRow+iPositionOfHighLightedItem, iLCol+1 SAY ;
      aPromptsForPullDown[iPositionOfHighLightedItem] ;
      COLOR IF( ITEM_IS_ACTIVE, HIGHLIGHT, [N+/W])

   IF !(ITEM_IS_ACTIVE)

      @ MAXROW(),10 SAY ;
         PADC([Selection NOT Currently Available!],60) COLOR MSG_COLOR

   ELSEIF !(EMPTY(aMsgsForPullDown[iPositionOfHighLightedItem]))

      @ MAXROW(),10 SAY ;
         PADC(aMsgsForPullDown[iPositionOfHighLightedItem],60);
         COLOR MSG_COLOR
   ELSE

      @ MAXROW(),10 SAY SPACE(60) COLOR MSG_COLOR

   ENDIF
   DISPEND()
RETURN(NIL)



#define  SUB_MENU_STRUCTURE   aMenu
#define  SUB_MENU_SAVESCREEN  aMenu[1]
#define  SUB_MENU_HEIGHT      aMenu[2]
#define  SUB_MENU_WIDTH       aMenu[3]
#define  SUB_MENU_PROMPTS     aMenu[4]
#define  SUB_MENU_ACTIONS     aMenu[5]
#define  SUB_MENU_MSGS        aMenu[6]
#define  SUB_MENU_HOTKEYS     aMenu[7]
#define  SUB_MENU_SEQ_NBRS    aMenu[8]


*  Function Ŀ
*          Name: SubChoice()           Docs: Steven Gibb                    
*   Description: Function to use in text menu definition file to chain      
*                a menu item action to the execution call of a submenu. It  
*                also processes keystrokes for the submenu level including  
*                executing actions if submenu item is selected. It is used  
*                recursively to allow chaining more than one level of       
*                submenu depth.                                             
*        Author: Steven Gibb                                                
*  Date created: 09-15-93              Date updated: 09-15-93              
*  Time created: 01:17:15pm            Time updated: 01:17:15pm            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: cNameOfSubmenu                                             
*  Return Value: (NIL)                                                      
*      See Also:                                                            
* 
FUNCTION SubChoice( cNameOfSubmenu )
   LOCAL iKey,cKey,iLCol,iTRow,iLastItem,iElementNbr,;
         aMenu,iSelectPos,iBRow,iRCol, cStr


   SUB_LEVEL_DEPTH++
   PushScreen()
   BEGIN SEQUENCE

      // scan the array that stores arrays of menu structures to
      // locate the submenu requested by name ( in character parameter
      // passed to this function )

      iElementNbr := ASCAN(MENU_ARRAY,{|x| cNameOfSubmenu $ x[1] } )

      IF iElementNbr == 0  // submenu structure found
         BREAK
      ENDIF

      SUB_MENU_STRUCTURE := MENU_ARRAY[iElementNbr][2]

      iLCol := LEFT_COL[TOP_BAR_POSITION];
         +(SUB_LEVEL_DEPTH * IF( LEFT_COL[TOP_BAR_POSITION] < 40, 4, -4 ))
      iTRow := 4 * SUB_LEVEL_DEPTH
      iRCol := iLCol + SUB_MENU_WIDTH
      iBRow := iTRow + SUB_MENU_HEIGHT

      iSelectPos := 1
      iLastItem := LEN(aMenu[4])
      DrawPullDownMenu( iTRow,iLCol,iBRow,iRCol,;
         SUB_MENU_SAVESCREEN,;
         iSelectPos, ;
         SUB_MENU_PROMPTS,;
         SUB_MENU_MSGS,;
         SUB_MENU_SEQ_NBRS)

      WHILE TRUE

         iKey := InKeyWait(0)

         IF iKey == K_RIGHT
            cStr := CHR(K_ESC) + CHR(K_RIGHT)
            KEYBOARD cStr
         ELSEIF iKey == K_LEFT
            cStr := CHR(K_ESC) + CHR(K_LEFT)
            KEYBOARD cStr
         ENDIF

         cKey := UPPER( CHR(iKey))
         DO CASE
            CASE iKey == K_HOME
               iSelectPos := 1
            CASE iKey == K_END
               iSelectPos := iLastItem
            CASE iKey == K_UP
               IF iSelectPos == 1
                  IF MENU_WRAP
                     iSelectPos := iLastItem
                  ENDIF
               ELSE
                  iSelectPos--
               ENDIF
            CASE iKey == K_DOWN
               IF iSelectPos == iLastItem
                  IF MENU_WRAP
                     iSelectPos := 1
                  ENDIF
               ELSE
                  iSelectPos++
               ENDIF

            CASE cKey $ SUB_MENU_HOTKEYS
               iSelectPos := AT( cKey, SUB_MENU_HOTKEYS )

               DrawPullDownMenu( iTRow,iLCol,iBRow,iRCol,;
                  SUB_MENU_SAVESCREEN,;
                  iSelectPos,;
                  SUB_MENU_PROMPTS,;
                  SUB_MENU_MSGS,;
                  SUB_MENU_SEQ_NBRS)

               IF ITEM_IS_ACTIVE
                  EVAL( SUB_MENU_ACTIONS[iSelectPos])
               ENDIF

            CASE iKey == K_RETURN
               IF ITEM_IS_ACTIVE
                  EVAL( SUB_MENU_ACTIONS[iSelectPos])
               ENDIF

            CASE iKey == K_ESC
               EXIT

         ENDCASE
         IF iSelectPos > 0
            DrawPullDownMenu( iTRow,iLCol,iBRow,iRCol,;
               SUB_MENU_SAVESCREEN,;
               iSelectPos,;
               SUB_MENU_PROMPTS,;
               SUB_MENU_MSGS,;
               SUB_MENU_SEQ_NBRS)
         END
      ENDDO
   END SEQUENCE
   PopScreen()
   SUB_LEVEL_DEPTH--
RETURN(NIL)

*  Function Ŀ
*          Name: InitMenu()            Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:14:02am            Time updated: 07:14:02am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: cFile                                                      
*  Return Value: ( MENU_STRUCT )                                            
*      See Also:                                                            
* 
FUNCTION InitMenu( cFile )
   LOCAL k, cTmp, iItemCol, iPos, iLastEle, aSubmenu, i
   LOCAL aTopLine   := {}
   LOCAL aTmp       := {}
   LOCAL aSubs      := {}
   LOCAL iNbrSelections := 0
   SET CURSOR OFF
   PushScreen()
   DISPBEGIN()

   MENU_STRUCT       := ARRAY( ARRAY_SIZE )
   LEFT_COL          := {}
   BAR_HOTKEYS       := []
   BAR_CHOICES       := {}
   SPACE_BETWEEN     := 2
   BAR_PAD           := 2
   START_COL         := 4
   MENU_WRAP         := TRUE
   MENU_COLOR        := [N/BG]
   HIGHLIGHT         := [W+/R]
   MSG_COLOR         := [GR+/B]
   HOTKEY_COLOR      := [R/BG]
   BACKGROUND_COLOR  := [W/B]
   SHADOW            := TRUE
   MENU_MONO         := [N/W]
   HIGHLT_MONO       := [W/N]
   MSG_MONO          := [W/N]
   HOTKEY_MONO       := [W+/N]
   BACKGROUND_MONO   := [W/N]

   cTmp :=  MEMOREAD(cFile)

   WHILE !EMPTY(cTmp)
      iPos := AT(CHR(13)+CHR(10), cTmp)-1
      AADD(aTmp, SUBSTR(cTmp,1,iPos) )
      IF [#####] $ ATAIL( aTmp)
         EXIT
      ENDIF
      cTmp := SUBSTR(cTmp,iPos+3)
   END

   k := 1
   WHILE SUBS(aTmp[k],1,1) <> [@]
      k++
   ENDDO
   IF k > 1
      SetMenuAttributes(k,aTmp)
   ENDIF
   NBR_MENUS := 0

   CheckColor()

   WHILE SUBS(aTmp[k],1,1) == [@]
      IF SUBS(aTmp[k],2,1) <> [@]
         NBR_MENUS++
         AADD( aTopLine, STRTRAN(aTmp[k],[@]))
      ENDIF

      AADD( aSubs, {})
      iLastEle := LEN( aSubs )
      cTmp := STRTRAN( aTmp[k],[~])
      cTmp := STRTRAN( cTmp,[@])
      AADD( aSubs[iLastEle], TRIM( cTmp ))       // insert name of submenu

      k++
      aSubMenu := {}
      WHILE SUBS(aTmp[k],1,1) == [|]
         AADD( aSubMenu, aTmp[k] )
         k++
         iNbrSelections++
      ENDDO
      AADD( aSubs[iLastEle], BuildSubMenu(aSubMenu) )
   ENDDO
   MENU_ARRAY := aSubs
   @ 00,00 SAY SPACE( MAXCOL()+1 ) COLOR MENU_COLOR
   iItemCol := START_COL
   FOR k = 1 TO NBR_MENUS
      cTmp             := SPACE(BAR_PAD)+aTopLine[k]+SPACE(BAR_PAD)
      iPos             := AT([~],cTmp )
      cTmp             := STRTRAN( cTmp,[~])
      BAR_HOTKEYS      += UPPER(SUBS( cTmp, iPos,1))
      AADD( BAR_CHOICES, cTmp )
      AADD( LEFT_COL, iItemCol )
      iItemCol         += LEN( cTmp )+SPACE_BETWEEN
      @ 00, LEFT_COL[k] SAY  cTmp  COLOR MENU_COLOR
      @ 00, LEFT_COL[k]+iPos-1 SAY SUBSTR( cTmp,iPos,1 ) COLOR HOTKEY_COLOR
   NEXT
   BAR_MENU_SCREEN := SAVESCREEN(00,00,00,MAXCOL() )
   PopScreen()
   DISPEND()
   ASIZE(ITEMS_ACTIVE_STATUS, iNbrSelections)
   AFILL(ITEMS_ACTIVE_STATUS, TRUE )

RETURN( MENU_STRUCT )

*  Function Ŀ
*          Name: SetItemStatus()       Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 08:34:22am            Time updated: 08:34:22am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: aItemNbrs                                                  
*              : lNewValue                                                  
*  Return Value: ( NIL )                                                    
*      See Also:                                                            
* 
FUNCTION SetItemStatus( aItemNbrs, lNewValue )
   LOCAL   i
   LOCAL   k := LEN( aItemNbrs  )
   FOR i := 1 TO k
      ITEMS_ACTIVE_STATUS[ aItemNbrs[i] ] := lNewValue
   NEXT
RETURN( NIL )



*  Function Ŀ
*          Name: CheckColor()          Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:14:44am            Time updated: 07:14:44am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: None                                                       
*  Return Value: (NIL)                                                      
*      See Also:                                                            
* 
STATIC FUNCTION CheckColor()
   IF !ISCOLOR()
      MENU_COLOR       :=  MENU_MONO
      BACKGROUND_COLOR :=  BACKGROUND_MONO
      HIGHLIGHT        :=  HIGHLT_MONO
      MSG_COLOR        :=  MSG_MONO
      HOTKEY_COLOR     :=  HOTKEY_MONO
   ENDIF
RETURN(NIL)

*  Function Ŀ
*          Name: BuildSubMenu()        Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:15:08am            Time updated: 07:15:08am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: aItems                                                     
*  Return Value: RETURN( aSubMenu )                                         
*      See Also:                                                            
* 
FUNCTION BuildSubMenu( aItems )
   LOCAL k, iHotKeyPos, iPos, cItem, aMenuItemElements, iSubWidth
   LOCAL iSubHeight   := LEN(aItems)
   LOCAL aSubActions  := {}
   LOCAL aSubPrompts  := {}
   LOCAL aSubMsgs     := {}
   LOCAL cHotKeys     := []
   LOCAL aSubMenu     := {}
   LOCAL aItemSeqNbr   := {}

   FOR k := 1 TO iSubHeight
      aMenuItemElements := ParseMenuLine( aItems[k])
      cItem := SPACE(1)+aMenuItemElements[1]+ ;
         IF([SubChoice(]$aMenuItemElements[3],[],[ ])
      iHotKeyPos := AT([~], cItem )
      cHotKeys += UPPER(SUBSTR( cItem, iHotKeyPos+1,1))
      cItem := STRTRAN( cItem,[~] )
      @ k,1 SAY cItem COLOR MENU_COLOR
      @ k,iHotKeyPos SAY SUBSTR(cItem,iHotKeyPos,1) COLOR HOTKEY_COLOR
      AADD( aSubPrompts, cItem  )
      AADD( aSubActions, &("{||"+aMenuItemElements[3]+"}" ) )
      AADD( aItemSeqNbr, aMenuItemElements[4] )

      AADD( aSubMsgs, aMenuItemElements[2]+SPACE(1)    )
   NEXT
   iSubWidth := LEN( cItem )
   @ 00,00,iSubHeight+1,iSubWidth+1 BOX [Ŀ] COLOR MENU_COLOR
   AADD( aSubMenu, SAVESCREEN(00,00,iSubHeight+1,iSubWidth+1))
   AADD( aSubMenu, iSubHeight+1)
   AADD( aSubMenu, iSubWidth+1)
   AADD( aSubMenu, aSubPrompts)
   AADD( aSubMenu, aSubActions)
   AADD( aSubMenu, aSubMsgs)
   AADD( aSubMenu, cHotKeys)
   AADD( aSubMenu, aItemSeqNbr)
RETURN( aSubMenu )

*  Function Ŀ
*          Name: ParseMenuLine()       Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 08:10:51pm            Time updated: 08:10:51pm            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: cStr                                                       
*  Return Value: ( aMenuItemElements )                                      
*      See Also:                                                            
* 
STATIC FUNCTION ParseMenuLine( cStr )
   LOCAL aMenuItemElements := ARRAY(4)
   cStr :=  SUBSTR( cStr, 2, RAT([|],cStr)-2)    // strip | off each end
   aMenuItemElements[1] := SUBSTR( cStr,1,AT([|],cStr)-1)
   cStr := SUBSTR( cStr,AT([|],cStr)+1)
   aMenuItemElements[2] := ALLTRIM(SUBSTR( cStr,1,AT([|],cStr)-1))
   cStr := SUBSTR( cStr,AT([|],cStr)+1)
   aMenuItemElements[3] := SUBSTR( cStr,1,AT([|],cStr)-1)
   aMenuItemElements[4] := VAL( SUBSTR( cStr,AT([|],cStr)+1) )
RETURN( aMenuItemElements )

*  Function Ŀ
*          Name: SetMenuAttributes()   Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:15:39am            Time updated: 07:15:39am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: k                                                          
*              : aTmp                                                       
*  Return Value: RETURN(NIL)                                                
*      See Also:                                                            
* 
FUNCTION SetMenuAttributes(k,aTmp)
   LOCAL i, cTmp1, cTmp2
   FOR i := 1 TO k
      cTmp1 := LOWER(SUBSTR(aTmp[i],1,AT([=],aTmp[i])-1))
      cTmp2 := LOWER(SUBSTR(aTmp[i],AT([=],aTmp[i])+1))
      DO CASE
         CASE cTmp1 == [space_between]
            SPACE_BETWEEN :=  INT(VAL( cTmp2 ))
         CASE cTmp1 == [bar_pad]
            BAR_PAD :=  INT(VAL( cTmp2 ))
         CASE cTmp1 == [start_col]
            START_COL :=  INT(VAL( cTmp2 ))
         CASE cTmp1 == [menu_wrap]
            MENU_WRAP := IF(ALLTRIM(cTmp2)==[yes], TRUE, FALSE )
         CASE cTmp1 == [menu_color]
            MENU_COLOR := ALLTRIM( cTmp2)
         CASE cTmp1 == [highlight]
            HIGHLIGHT := ALLTRIM( cTmp2)
         CASE cTmp1 == [msg_color]
            MSG_COLOR := ALLTRIM( cTmp2)
         CASE cTmp1 == [hotkey_color]
            HOTKEY_COLOR := ALLTRIM( cTmp2)
         CASE cTmp1 == [background_color]
            BACKGROUND_COLOR := ALLTRIM( cTmp2)

         CASE cTmp1 == [menu_mono]
            MENU_MONO := ALLTRIM( cTmp2)
         CASE cTmp1 == [highlt_mono]
            HIGHLT_MONO := ALLTRIM( cTmp2)
         CASE cTmp1 == [msg_mono]
            MSG_MONO := ALLTRIM( cTmp2)
         CASE cTmp1 == [hotkey_mono]
            HOTKEY_MONO := ALLTRIM( cTmp2)
         CASE cTmp1 == [background_mono]
            BACKGROUND_MONO := ALLTRIM( cTmp2)
         CASE cTmp1 == [shadow]
            SHADOW := IF(ALLTRIM(cTmp2)==[yes], TRUE , FALSE )
      ENDCASE
   NEXT
RETURN(NIL)




*  Function Ŀ
*          Name: Stub()                Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:16:44am            Time updated: 07:16:44am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: cStub                                                      
*  Return Value: (NIL)                                                      
*      See Also:                                                            
* 
FUNCTION Stub( cStub )
   PushScreen()
   DrawBox( 15, 20, 19, 60, 1, 1, [w/r], [Stub])
   @ 16,22 SAY [    INKEY() value of Keystroke: ] + ALLTRIM( STR(LASTKEY(),3) ) COLOR [w+/r]
   @ 17,22 SAY [      CHR() value of Keystroke: ] + CHR(LASTKEY()) COLOR [w+/r]
   @ 18,25 SAY TRIM(cStub)+[ will hook in here.] COLOR [w+/r]
   @ 24,00 SAY PADC([Press any key to continue.],MAXCOL()) COLOR [GR+/B]
   inkey(0)
   PopScreen()
RETURN(NIL)


*  Function Ŀ
*          Name: BasicScreen()         Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:16:51am            Time updated: 07:16:51am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: None                                                       
*  Return Value: ( NIL )                                                    
*      See Also:                                                            
* 
FUNCTION BasicScreen()
   STATIC cScreen
   IF cScreen == NIL
      SETCOLOR( IF( ISCOLOR(),[W/B,N/W,,,W/B], [W]) )
      @ 01,00,MAXROW() -2,MAXCOL() BOX []
      @ MAXROW()-1,00 SAY   REPLICATE( [], MAXCOL()+1 )
      ClearMsg()
      cScreen := SAVESCREEN(01,00,MAXROW(),MAXCOL())
   ELSE
      RESTSCREEN(01,00,MAXROW(),MAXCOL(),cScreen)
   ENDIF
RETURN( NIL )

*  Function Ŀ
*          Name: ClearMsg()            Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 07:16:56am            Time updated: 07:16:56am            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: None                                                       
*  Return Value: ( .T. )                                                    
*      See Also:                                                            
* 
FUNCTION ClearMsg()
   @ MAXROW(), 00 SAY SPACE(MAXCOL()+1) COLOR  [GR+/B]
RETURN( TRUE )



*  Function Ŀ
*          Name: InKeyWait()           Docs: Steven Gibb                    
*   Description: Similar to INKEY(), except services SET KEYs                                                           
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 08:01:54pm            Time updated: 08:01:54pm            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: nSecs                                                      
*  Return Value: ( nKey )                                                 
*      See Also:                                                            
* 
FUNCTION InKeyWait( nSecs )
   LOCAL nKey
   LOCAL bKeyBlock

   nKey := INKEY( nSecs )

   IF (bKeyBlock := SETKEY(nKey)) != NIL
      EVAL(bKeyBlock, PROCNAME(2), PROCLINE(2))
   ENDIF

RETURN( nKey )



*  Function Ŀ
*         Name: PushScreen()          Docs: Steven Gibb                     
*  Description: Saves the CHR() value of the current cursor shape, cursor   
*               row, cursor column, top row, left column, bottom row,       
*               right column as the first seven characters of a string.     
*               The return of a SAVESCREEN() for the part of the screen     
*               to be saved is concatenated to the string and then the      
*               string is pushed on the screen stack.                       
*               The numbers are always less than 255 so they are converted  
*               to CHR() values allowing them to be stored as characters in 
*               in one byte.  If screen coordinates are not passed          
*               as parameters, the function defaults to pushing a full      
*               screen.                                                     
*                                                                           
*  Notes: There is no error checking on the parameters. Must pass either    
*         no parameters or four valid screen coordinate integer values!     
*                                                                           
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 08:02:42pm            Time updated: 08:02:42pm            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: t   top row of area to be saved on stack                   
*              : l   left column of area to be saved on stack               
*              : b   bottom row of area to be saved on stack                
*              : r   right column of area to be saved on stack              
*  Return Value: ( NIL )                                                    
*      See Also:                                                            
* 
FUNCTION PushScreen(t,l,b,r)

   IF t==NIL                                     // if parameter missing set default to full screen
      t := 0
      l := 0
      b := MAXROW()
      r := MAXCOL()
   ENDIF

   iScreenStackNbr++                             // increment pointer to stack top

   AADD( aScreenStack, CHR( SETCURSOR() );       // cursor shape
      +CHR( ROW() );                             // cursor row
      +CHR( COL() );                             // cursor column
      +CHR(t)+CHR(l)+CHR(b)+CHR(r);              // screen coordinates
      +SAVESCREEN(t,l,b,r) )                     // screen string

RETURN( NIL )


*  Function Ŀ
*          Name: PopScreen()           Docs: Steven Gibb                    
*   Description: The current cursor shape, cursor row, cursor column,       
*                top row, left column, bottom row, right column and screen  
*                are retrieved from a character string that was the last    
*                element pushed on the screen stack.                        
*                Cursor shape, cursor row, cursor column, top row,          
*                left column, bottom row and right column are extracted from
*                the first seven positions in the string and converted back 
*                to numbers before restoring the cursor and the screen.     
*                                                                           
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 08:07:37pm            Time updated: 08:07:37pm            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: None                                                       
*  Return Value: ( NIL )                                                  
*      See Also:                                                            
* 
FUNCTION PopScreen()
   LOCAL iCurShape := ASC( SUBSTR( aScreenStack[iScreenStackNbr],1,1))
   LOCAL iDevRow   := ASC( SUBSTR( aScreenStack[iScreenStackNbr],2,1))
   LOCAL iDevCol   := ASC( SUBSTR( aScreenStack[iScreenStackNbr],3,1))
   LOCAL t         := ASC( SUBSTR( aScreenStack[iScreenStackNbr],4,1))
   LOCAL l         := ASC( SUBSTR( aScreenStack[iScreenStackNbr],5,1))
   LOCAL b         := ASC( SUBSTR( aScreenStack[iScreenStackNbr],6,1))
   LOCAL r         := ASC( SUBSTR( aScreenStack[iScreenStackNbr],7,1))
   LOCAL cScreen   := SUBSTR( aScreenStack[iScreenStackNbr],8)

   SETCURSOR( iCurShape )                        // set cursor shape

   DEVPOS( iDevRow, iDevCol )                    // set cursor location

   RESTSCREEN(t,l,b,r, cScreen)                  // restore screen

   iScreenStackNbr--                             // decrement top of stack
   // pointer

   ASize( aScreenStack, iScreenStackNbr )        // reduce stack size by 1

RETURN( NIL )


*  Function Ŀ
*          Name: QuitProgram()         Docs: Steven Gibb                    
*   Description:                                                            
*        Author: Steven Gibb                                                
*  Date created: 09-21-93              Date updated: 09-21-93              
*  Time created: 08:10:07pm            Time updated: 08:10:07pm            
*     Copyright: (c) 1993 by Steven Gibb                                    
* Ĵ
*     Arguments: None                                                       
*  Return Value: (NIL)                                                      
*      See Also:                                                            
* 
FUNCTION QuitProgram()
   SETCOLOR([W/B])
   CLEAR SCREEN
   CLOSE ALL
   QUIT
RETURN(NIL)


